import { reactive, ref } from 'vue';
import { currentMouth, days } from '@/enums/dateEnums';
import { fetchHomeDatalist } from '@/api/v1/system/system';
import dayjs from 'dayjs';
import { $t } from '@/plugins/i18n';

// 支出百分比
export const expendPercent = ref(0);
// 顶部图表数据
export const chartData = ref([]);
// 收入支出排行榜
export const homeRanks = ref([]);
// 分析概览x轴
export const xAxis = ref([]);
// 分析概览收入
export const incomeData = ref([]);
// 分析概览支出
export const expendData = ref([]);
// 查询日期表单
export const form = reactive({
	startDate: undefined,
	endDate: undefined,
	dateRange: currentMouth,
});

/** 初始化数据 */
export const onSearch = async () => {
	// 初始化请求数据，设置开始时间和结束时间范围
	form.startDate = dayjs(form.dateRange[0]).format('YYYY-MM-DD');
	form.endDate = dayjs(form.dateRange[1]).format('YYYY-MM-DD');

	// 请求后端数据并判断是否成功
	const result = await fetchHomeDatalist(form);
	if (result.code !== 200) return;

	chartData.value = [];
	const homeCard = result.data?.homeCard;
	const income = homeCard?.income?.amount ? homeCard?.income?.amount : 0;
	const expend = homeCard?.expend?.amount ? homeCard?.expend?.amount : 0;

	// 支出百分比
	const expendPercentValue = Number((expend / (income + expend)).toFixed(2)) * 100;
	expendPercent.value = expendPercentValue ? expendPercentValue : 0;

	/* 收入和支出分析概览 */
	xAxis.value = result.data.homeOverview
		.filter(item => item.type === 1)
		.map(item => {
			const transactionDate = item.transactionDate;
			return `${dayjs(transactionDate).format('MM-DD')} ${days[dayjs(transactionDate).day()]}`;
		});
	incomeData.value = result.data.homeOverview.filter(item => item.type === 1).map(item => item.amount);
	expendData.value = result.data.homeOverview.filter(item => item.type === -1).map(item => item.amount);

	/* 收入和支出排行榜 */
	homeRanks.value = result.data.homeRanks.map(item => {
		const transactionDate = item.transactionDate;
		const date = `${dayjs(transactionDate).format('YYYY-MM-DD')} ${days[dayjs(transactionDate).day()]}`;
		return { date, amount: item.amount, name: item.categoryName, type: item.type };
	});

	/* 顶部卡片数据 */
	homeCardFun(income, expend);
};

/**
 * 顶部卡片初始化数据
 * @param income 收入
 * @param expend 支出
 */
function homeCardFun(income, expend) {
	/* 收入金额 */
	let data = {
		icon: 'icon-park-outline:income',
		bgColor: '#effaff',
		color: '#67C23A',
		duration: 2200,
		name: $t('totalIncomeOfTheMonth'),
		value: income,
		percent: <p class='font-medium text-green-500'>{`+ ${income} ￥`}</p>,
		data: incomeData.value,
	};
	chartData.value.push(data);

	/* 支出金额 */
	data = {
		icon: 'icon-park-outline:expenses',
		bgColor: '#fff5f4',
		color: '#F56C6C',
		duration: 2200,
		name: $t('totalExpendituresOfTheMonth'),
		value: expend,
		percent: <p class='font-medium text-red-500'>{`- ${expend} ￥`}</p>,
		data: expendData.value,
	};
	chartData.value.push(data);

	/* 盈利金额 */
	const profit = incomeData.value.map((value, index) => {
		const number = value - expendData.value[index];
		return isNaN(number) ? 0 : number;
	});
	const profitValue = (income - expend).toFixed(2);
	data = {
		icon: 'hugeicons:profit',
		bgColor: '#eef4ff',
		color: '#409EFF',
		duration: 2200,
		name: $t('amountOfProfit'),
		value: Number(profitValue),
		percent: <p class='font-medium text-blue-500'>{`${profitValue} ￥`}</p>,
		data: profit,
	};
	chartData.value.push(data);
}
